/*
 * Shared library for Tcl/Tk 8.0 to add postscript printing to
 * the canvas widget.
 *
 * Compile as(IRIX 6.4)
 *
 *       cc -c imagePs.c
 *       ld -shared -o libimagePs.so imagePs.o
 *
 * Please contact Daniel Blezek <blezek.daniel@mayo.edu>
 * with any questions.
 */

#include <tk.h>

typedef struct ImageItem  {
    Tk_Item header;             /* Generic stuff that's the same for all
                                 * types.  MUST BE FIRST IN STRUCTURE. */
    Tk_Canvas canvas;           /* Canvas containing the image. */
    double x, y;                /* Coordinates of positioning point for
                                 * image. */
    Tk_Anchor anchor;           /* Where to anchor image relative to
                                 * (x,y). */
    char *imageString;          /* String describing -image option (malloc-ed).
                                 * NULL means no image right now. */
    Tk_Image image;             /* Image to display in window, or NULL if
                                 * no image at present. */
} ImageItem;


static int ImageToPostscript _ANSI_ARGS_((Tcl_Interp *interp, Tk_Canvas canvas, Tk_Item *itemPtr, int prepass));

int AddImagePostscript () 
{
	Tk_ItemType* theItem;
	theItem = Tk_GetItemTypes();
	while ( theItem != NULL )
	{
		if ( strcmp ( theItem->name, "image" ) == 0 )
		{
			theItem->postscriptProc = ImageToPostscript;
			return TCL_OK;
		}
		theItem = theItem->nextPtr;
	}
	return TCL_ERROR;
}

int Imageps_Init ( Tcl_Interp* interp )
{
	return AddImagePostscript ();
}


static int ImageToPostscript ( interp, canvas, itemPtr, prepass )
    Tcl_Interp *interp;                 /* Leave Postscript or error message
                                         * here. */
    Tk_Canvas canvas;                   /* Information about overall canvas. */
    Tk_Item *itemPtr;                   /* Item for which Postscript is
                                         * wanted. */
    int prepass;                        /* 1 means this is a prepass to
                                         * collect font information;  0 means
                                         * final Postscript is being created. */
{
	ImageItem* item;
	int i, x, y, width, height;
	Tk_PhotoImageBlock block;
	Tk_PhotoHandle handle;
	unsigned char *red;
	unsigned char *green;
	unsigned char *blue;
	unsigned char it;
	char buffer[200];

	item = (ImageItem*) itemPtr;
	handle = NULL;
	if ( prepass )
	{
		return TCL_OK;
	}
	if ( item->image == NULL )
	{
		return TCL_OK;
	}

	if ( !( handle = Tk_FindPhoto ( interp, item->imageString ) ) )
	{
		return TCL_OK;
	}

	i = Tk_PhotoGetImage ( handle, &block );
	Tk_SizeOfImage ( item->image, &width, &height );
	x = item->x;
	y = Tk_CanvasPsY ( canvas, item->y );
	switch (item->anchor)
	{
		case TK_ANCHOR_NW:		y -= height;							break;
		case TK_ANCHOR_N:		x -= width/2.0;		y -= height;		break;
		case TK_ANCHOR_NE:		x -= width;			y -= height;		break;
		case TK_ANCHOR_E:		x -= width;			y -= height/2.0;	break;
		case TK_ANCHOR_SE:		x -= width;								break;
		case TK_ANCHOR_S:		x -= width/2.0;							break;
		case TK_ANCHOR_SW:												break;
		case TK_ANCHOR_W:		y -= height/2.0;						break;
		case TK_ANCHOR_CENTER:	x -= width/2.0;		y -= height/2.0;	break;
	};

	Tcl_AppendResult ( interp, "% This is the image: ", item->imageString, "\n", NULL );
	sprintf ( buffer, "%d %d translate\n", x, y  );
	Tcl_AppendResult ( interp, buffer, NULL );

	sprintf ( buffer, "%d %d scale\n", block.width, block.height );
	Tcl_AppendResult ( interp, "% Scale to proper size\n", buffer, NULL );

	sprintf ( buffer, "/%s %d string def\n", item->imageString, 3 * block.width );
	Tcl_AppendResult ( interp, buffer, NULL );

	sprintf ( buffer, "%d %d 8\n[%d 0 0 -%d 0 %d]",
	block.width, block.height, block.width, block.height, block.height );
	Tcl_AppendResult ( interp, buffer, "\n{ currentfile ", item->imageString, " readhexstring pop }\nfalse 3 colorimage\n", NULL );

	red = block.pixelPtr + block.offset[0];
	green = block.pixelPtr + block.offset[1];
	blue = block.pixelPtr + block.offset[2];

	for ( i = 0; i < block.width * block.height; i++ )
	{
		/* For each, put out the hex value... */
		sprintf ( buffer, "%02x%02x%02x", *red, *green, *blue );
		red += block.pixelSize;
		green += block.pixelSize;
		blue += block.pixelSize;
		Tcl_AppendResult ( interp, buffer, NULL );
		if ( (i % 25 ) == 0 )
		{ 
			/*
			 * Add a return every 25 pixels.
			 */
			Tcl_AppendResult ( interp, "\n", NULL );
		}
	}

	return TCL_OK;
}
 





